/*
	libblockdev.c
	Simple functionality for startup, mounting and unmounting of raw devices.
	
 Copyright (c) 2007 SukkoPera, based on work by Michael "Chishm" Chisholm.
	
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation and/or
     other materials provided with the distribution.
  3. The name of the author may not be used to endorse or promote products derived
     from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	2007-01-08 - SukkoPera
		* Original release
*/

#include <sys/iosupport.h>

#include "rawio.h"
#include "common.h"
#include "rawpartition.h"

#define GBA_DEFAULT_CACHE_PAGES 2
#define NDS_DEFAULT_CACHE_PAGES 8


const devoptab_t dotab_rawio = {
	RAW_DEVICE_NAME,
	sizeof (RAWIO_STRUCT),
	_RAWIO_open_r,
	_RAWIO_close_r,
	_RAWIO_write_r,
	_RAWIO_read_r,
	_RAWIO_seek_r,
	_RAWIO_fstat_r,
	_RAWIO_stat_r,
	_RAWIO_link_r,
	_RAWIO_unlink_r,
	_RAWIO_chdir_r
};

bool rawioInit (bool setAsDefaultDevice) {
	bool out;
#ifdef NDS
	bool slot1Device, slot2Device;
	
	// Try mounting both slots
	slot1Device = _RAWIO_partition_mount (PI_SLOT_1);
	slot2Device = _RAWIO_partition_mount (PI_SLOT_2);
	
	// Choose the default device
	if (slot1Device) {
		_RAWIO_partition_setDefaultInterface (PI_SLOT_1);
	} else if (slot2Device) {
		_RAWIO_partition_setDefaultInterface (PI_SLOT_2);
	} else {
		return false;
	}

#else	// not defined NDS
	bool cartSlotDevice;

	cartSlotDevice = _RAWIO_partition_mount (PI_CART_SLOT);
	
	if (cartSlotDevice) {
		_RAWIO_partition_setDefaultInterface (PI_CART_SLOT);
	} else {
		return false;
	}	
	
#endif	// defined NDS

	if (AddDevice (&dotab_rawio) >= 0) {
		if (setAsDefaultDevice) {
			setDefaultDevice (FindDevice (RAW_DEVICE_NAME ":"));
		}
		out = true;
	} else {
		/* Error adding device */
		out = false;
	}
	
	return out;
}

bool rawioInitDefault (void) {
	return rawioInit (true);
}


bool rawioMountNormalInterface (PARTITION_INTERFACE partitionNumber) {
	return _RAWIO_partition_mount (partitionNumber);
}

#if 0
bool fatMountCustomInterface (const IO_INTERFACE* device, u32 cacheSize) {
	return _FAT_partition_mountCustomInterface (device);
}
#endif

bool rawioUnmount (PARTITION_INTERFACE partitionNumber) {
	return _RAWIO_partition_unmount (partitionNumber);
}


bool rawioUnsafeUnmount (PARTITION_INTERFACE partitionNumber) {
	return _RAWIO_partition_unsafeUnmount (partitionNumber);
}


#if 0
bool fatSetDefaultInterface (PARTITION_INTERFACE partitionNumber) {
	return _FAT_partition_setDefaultInterface (partitionNumber);
}
#endif
